@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------
-# Copyright 2006-2020 UT-Battelle, LLC. See LICENSE for more information.
+# Copyright 2006-2021 UT-Battelle, LLC. See LICENSE for more information.
 # -------------------------------------------------------------------------------
 """
 The CCA event service, but without CCA-specific extensions:
@@ -86,8 +86,6 @@ class EventListener:
         method, but currently serves to check the correct operation of the
         event service.
         """
-        # TODO: To be deleted#
-        _proxy._processEvent(self.listenerid, topicName, theEvent)
 
 
 class Topic:
