@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------
-# Copyright 2006-2020 UT-Battelle, LLC. See LICENSE for more information.
+# Copyright 2006-2021 UT-Battelle, LLC. See LICENSE for more information.
 # -------------------------------------------------------------------------------
 import os
 import sys
@@ -61,7 +61,7 @@ class Driver(Component):
         listener = Listener(str(self.socket_address), 'AF_UNIX')
         self.services.warning('Created listener %s', str(self.socket_address))
         print('%s  Created Listener %s' % (
-            time.strftime("%b %d %Y %H:%M:%S", time.localtime()), str(self.socket_address)), str(listener))
+            time.strftime("%b %d %Y %H:%M:%S", time.localtime()), str(self.socket_address)))
         sys.stdout.flush()
         sim_cache = {}
         sock_fileno = listener._listener._socket.fileno()
@@ -141,12 +141,11 @@ class Driver(Component):
             self.old_master_conf['OUT_REDIRECT_FNAME'] = fname
             print('Redirecting stdout for %s to %s ' % (self.old_master_conf['SIM_NAME'], fname))
             try:
-                os.makedirs(self.old_master_conf['SIM_ROOT'])
+                os.makedirs(self.old_master_conf['SIM_ROOT'], exist_ok=True)
             except OSError as oserr:
-                if oserr.errno != 17:
-                    print('Error creating Simulation directory %s : %d %s' %
-                          (self.old_master_conf['SIM_ROOT'], oserr.errno, oserr.strerror))
-                    raise
+                print('Error creating Simulation directory %s : %d %s' %
+                      (self.old_master_conf['SIM_ROOT'], oserr.errno, oserr.strerror))
+                raise
             if first_sim:
                 summary_file = open(os.path.join(self.sim_root, 'SIMULATION_LIST.%s' % (dakota_runid)), 'a', 1)
 
