@@ -1,6 +1,6 @@
 #! /usr/bin/env python3
 # -------------------------------------------------------------------------------
-# Copyright 2006-2020 UT-Battelle, LLC. See LICENSE for more information.
+# Copyright 2006-2021 UT-Battelle, LLC. See LICENSE for more information.
 # -------------------------------------------------------------------------------
 """
    The Integrated Plasma Simulator (IPS) Framework. This framework enables
@@ -36,7 +36,7 @@
       R. Bramley, J. Breslau, M. Chance, J. Chen, M. Choi, W. Elwasif,
       G. Fu, R. Harvey, E. Jaeger, S. Jardin, T. Jenkins, D. Keyes,
       S. Klasky, S. Kruger, L. Ku, V. Lynch, D. McCune, J. Ramos, D. Schissel,
-      D. Schnack, and J. Wright, *Simulation of Wave Interations with MHD*,
+      D. Schnack, and J. Wright, *Simulation of Wave Interactions with MHD*,
       in Rick Stevens, editor, SciDAC 2008, 14-17 July 2008, Washington, USA,
       volume 125 of Journal of Physics: Conference Series, page 012039,
       Institute of Physics, 2008.
@@ -87,7 +87,7 @@ class Framework:
       * Manage communication queues, and route service requests from simulation
         components to appropriate managers.
       * Provide logging services to IPS managers.
-      * Perfrom shutdown procedure on exit
+      * Perform shutdown procedure on exit
 
     :param config_file_list: A list of simulation configuration files to be used
             in the simulaion. Each simulation configuration file must have the following
@@ -125,6 +125,9 @@ class Framework:
                  debug=False, verbose_debug=False, cmd_nodes=0, cmd_ppn=0):
         # added compset_list for list of components to load config files for
         # command line option
+        print("Starting IPS", get_versions()['version'])
+        os.environ['IPS_INITIAL_CWD'] = os.getcwd()
+
         self.log_file_name = log_file_name
         if log_file_name == 'sys.stdout':
             self.log_file = sys.stdout
@@ -273,7 +276,6 @@ class Framework:
         response_q = self.comp_registry.getComponentArtifact(comp_id,
                                                              'svc_response_q')
         response_q.put(response_msg)
-        return
 
     def log(self, msg, *args):
         """
@@ -452,7 +454,7 @@ class Framework:
                         outstanding_sim_calls[sim_name] = msg_list
 
         except Exception:
-            self.exception('encountered exception during fwk.run() genration of call messages')
+            self.exception('encountered exception during fwk.run() generation of call messages')
             self.terminate_all_sims(status=Message.FAILURE)
             return False
 
@@ -602,6 +604,7 @@ class Framework:
                 pass
             portal_data['startat'] = time.strftime('%Y-%m-%d|%H:%M:%S%Z',
                                                    time.localtime(self.start_time))
+            portal_data['ips_version'] = get_versions()['version']
         elif eventType == 'IPS_END':
             portal_data['state'] = 'Completed'
             portal_data['stopat'] = time.strftime('%Y-%m-%d|%H:%M:%S%Z',
@@ -678,6 +681,8 @@ def main(argv=None):
     """
     Check and parse args, create and run the framework.
     """
+    sys.stdout.flush()
+
     platform_default = os.environ.get("IPS_PLATFORM_FILE")
     if platform_default:
         print("IPS using platform file :", platform_default)
@@ -717,7 +722,4 @@ def main(argv=None):
 
 
 if __name__ == "__main__":
-    print("Starting IPS")
-    sys.stdout.flush()
-    os.environ['IPS_INITIAL_CWD'] = os.getcwd()
     sys.exit(main())
