@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------
-# Copyright 2006-2020 UT-Battelle, LLC. See LICENSE for more information.
+# Copyright 2006-2021 UT-Battelle, LLC. See LICENSE for more information.
 # -------------------------------------------------------------------------------
 
 
@@ -33,34 +33,6 @@ class IncompleteCallException(Exception):
         return 'nonblocking wait_call() invoked before call %s finished' % self.callID
 
 
-class AllocatedNodeDownException(Exception):
-    """ Exception is raised when an allocated node is discovered to be faulty.
-        The task manager should catch the exception and do something with it.
-    """
-
-    def __init__(self, identifier, tid, comp_id):
-        super().__init__()
-        self.node = identifier
-        self.tid = tid
-        self.component = comp_id
-        self.args = (identifier, tid, comp_id)
-
-    def __str__(self):
-        return 'allocated node {} has been deemed unfit. task {} of component {} needs to be killed and restarted.'.format(self.node, self.tid, self.component)
-
-
-class NonexistentResourceException(Exception):
-    """ Exception for any time nonexistent (nodes) are tried to be used
-    """
-
-    def __init__(self, identifier):
-        super().__init__()
-        self.thing = identifier
-
-    def __str__(self):
-        return self.thing + ' does not exist'
-
-
 class InsufficientResourcesException(Exception):
     """ Exception Raised by the resource manager when not enough resources
         are available to satisfy an allocate() call
@@ -119,8 +91,6 @@ class InvalidResourceSettingsException(Exception):
             return "%s socket per node count (%d) greater than core per node count (%d)." % (preamble, self.spn, self.cpn)
         elif self.type == "spn not divisible by cpn":
             return "%s socket per node count (%d) not divisible by core per node count (%d)." % (preamble, self.spn, self.cpn)
-        elif self.type == "total procs and nodes mismatch":
-            return "%s values for total procs (%d) and nodes (%d) do not make sense." % (preamble, self.spn, self.cpn)
         else:
             return "%s unknown error" % (preamble)
 
@@ -143,24 +113,3 @@ class BadResourceRequestException(Exception):
         return ("component " + str(self.caller_id) + " requested " + str(self.request) +
                 " nodes, which is more than possible by " + str(self.deficit) +
                 " nodes, for task " + str(self.task_id) + ".")
-
-
-class ReleaseMismatchException(Exception):
-    """ Exception raised by the resource manager when a release allocation
-        request accounting yields unexpected results.
-    """
-
-    def __init__(self, caller_id, tid, old_alc, old_avc, new_alc, new_avc):
-        super().__init__()
-        self.caller_id = caller_id
-        self.tid = tid
-        self.old_alc = old_alc
-        self.old_avc = old_avc
-        self.new_alc = new_alc
-        self.new_avc = new_avc
-        self.args = (caller_id, tid, old_alc, old_avc, new_alc, new_avc)
-
-    def __str__(self):
-        return ("component " + str(self.caller_id) + " released nodes for task " + str(self.tid) +
-                ".  old_alc = " + str(self.old_alc) + " new_alc = " + str(self.new_alc) +
-                "  old_avc = " + str(self.old_avc) + " new_avc = " + str(self.new_avc))
