@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 # -------------------------------------------------------------------------------
-# Copyright 2006-2020 UT-Battelle, LLC. See LICENSE for more information.
+# Copyright 2006-2021 UT-Battelle, LLC. See LICENSE for more information.
 # -------------------------------------------------------------------------------
 
 import os
@@ -166,12 +166,11 @@ IPS_ROOT/bin or IPS_ROOT/framework/src')
 
         sim_root = self.template_conf['SIM_ROOT']
         try:
-            os.makedirs(sim_root)
+            os.makedirs(sim_root, exist_ok=True)
         except OSError as oserr:
-            if oserr.errno != 17:
-                print('Error creating Simulation directory %s : %d %s' %
-                      (sim_root, oserr.errno, oserr.strerror))
-                raise
+            print('Error creating Simulation directory %s : %d %s' %
+                  (sim_root, oserr.errno, oserr.strerror))
+            raise
 
         for (k, v) in self.template_conf.items():
             if k not in list(self.master_conf.keys()):
@@ -210,7 +209,7 @@ IPS_ROOT/bin or IPS_ROOT/framework/src')
             cmd += '  --debug'
 
         print('cmd =', cmd)
-        ips_server_proc = subprocess.Popen(cmd, shell=True)
+        ips_server_proc = subprocess.Popen(cmd)
         print('%s  Launched IPS' % (time.strftime("%b %d %Y %H:%M:%S", time.localtime())))
         sys.stdout.flush()
         msg = {'SIMSTATUS': 'START'}
@@ -241,7 +240,7 @@ IPS_ROOT/bin or IPS_ROOT/framework/src')
         else:
             command = 'dakota %s ' % new_dakota_config
         dakota_logfile = open('dakota_%s.log' % (str(os.getpid())), 'w')
-        proc = subprocess.Popen(command, shell=True, stdout=dakota_logfile, stderr=subprocess.STDOUT)
+        proc = subprocess.Popen(command, stdout=dakota_logfile, stderr=subprocess.STDOUT)
         print('%s  Launched DAKOTA' % (time.strftime("%b %d %Y %H:%M:%S", time.localtime())))
         sys.stdout.flush()
         proc.wait()
