@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------
-# Copyright 2006-2020 UT-Battelle, LLC. See LICENSE for more information.
+# Copyright 2006-2021 UT-Battelle, LLC. See LICENSE for more information.
 # -------------------------------------------------------------------------------
 import re
 
@@ -26,7 +26,7 @@ mpo_cert = '/home/elwasif/Projects/atom/MPO/MPO Demo User.pem'
 mpo_api = 'https://mpo.psfc.mit.edu/test-api'
 
 
-def configure_mpo():
+def configure_mpo():  # pragma: no cover
     # Use this if you want to include modules from a subfolder or relative path.
     cmd_subfolder = os.path.realpath(os.path.abspath(os.path.join(os.path.split(
         inspect.getfile(inspect.currentframe()))[0], "/home/elwasif/Projects/atom/MPO/client/python")))
@@ -34,7 +34,7 @@ def configure_mpo():
         sys.path.insert(0, cmd_subfolder)
 
 
-def hash_file(file_name):
+def hash_file(file_name):  # pragma: no cover
     '''
     Return the MD5 hash of a file
     :rtype: str
@@ -119,13 +119,13 @@ class PortalBridge(Component):
         else:
             self.dump_freq = freq
 
-        try:
+        try:  # pragma: no cover
             ENABLE_MPO = os.environ['ENABLE_MPO']
         except KeyError:
             ENABLE_MPO = False
         else:
             ENABLE_MPO = True
-        if ENABLE_MPO:
+        if ENABLE_MPO:  # pragma: no cover
             configure_mpo()
             try:
                 self.mpo = mpo(api_url=mpo_api, cert=mpo_cert, debug=True)
@@ -169,15 +169,6 @@ class PortalBridge(Component):
             except Exception:
                 pass
 
-    def get_elapsed_time(self):
-        """
-        Return total elapsed time since simulation started in seconds
-        (including a possible fraction)
-        """
-        self.curTime = time.localtime()
-        delta_t = time.mktime(self.curTime) - time.mktime(self.startTime)
-        return delta_t
-
     def process_event(self, topicName, theEvent):
         """
         Process a single event *theEvent* on topic *topicName*.
@@ -206,7 +197,6 @@ class PortalBridge(Component):
         elif sim_data.monitor_url:
             portal_data['vizurl'] = sim_data.monitor_url
 
-        #        portal_data['walltime'] = int(self.get_elapsed_time())
         portal_data['portal_runid'] = sim_data.portal_runid
         portal_data['seqnum'] = sim_data.counter
 
@@ -224,7 +214,6 @@ class PortalBridge(Component):
             self.done = True
             self.services.debug('No more simulation to monitor - exiting')
             time.sleep(1)
-        return
 
     def send_event(self, sim_data, event_data):
         """
@@ -279,7 +268,7 @@ class PortalBridge(Component):
         if sim_data.mpo_wid:
             self.send_mpo_data(event_data, sim_data)
 
-    def send_mpo_data(self, event_data, sim_data):
+    def send_mpo_data(self, event_data, sim_data):  # pragma: no cover
         def md5(fname):
             "Courtesy of stackoverflow 3431825"
             hash_md5 = hashlib.md5()
@@ -315,7 +304,7 @@ class PortalBridge(Component):
             # api search results are sorted by time
             # Note, check this with eqdsk dataobject in test-api
             print('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
-            print(len(is_checksum), file, parent)
+            print(len(is_checksum), file)
             print('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
 
             if len(is_checksum) > 0:
@@ -325,7 +314,6 @@ class PortalBridge(Component):
 
                 # Create dataobject reference by uid in the workflow
                 dataobject = self.mpo.add(workflow, parent, uid=file_uid, name=shortname, desc=longdesc)
-                print('Existing dataobject : id = ', file_uid)
                 self.mpo.meta(dataobject['uid'], 'ips_checksum', checksum)
                 # add filename metadata the dataobject reference
                 self.mpo.meta(dataobject['uid'], 'ips_filename', file)
@@ -337,7 +325,6 @@ class PortalBridge(Component):
                 # add checksum metadata to original data object
                 # add function currently only returns uri field, so fetch full record
                 full_dataobject = self.mpo.search('dataobject/' + dataobject['uid'])[0]
-                print('New data object', full_dataobject['do_uid'])
                 # add checksum so dataobject and also
                 self.mpo.meta(full_dataobject['do_uid'], 'ips_checksum', checksum)
                 self.mpo.meta(dataobject['uid'], 'ips_filename', file)
@@ -345,24 +332,6 @@ class PortalBridge(Component):
                 dataobject = full_dataobject
             return dataobject
 
-        """
-        def get_file_uid(path, file_name):
-            '''
-            Get file_name's cached MPO uid if seen before in simulation
-            :param file_name: Full path to file
-            :return: tuple (MPO uid if found, None if not found, file_hash)
-            '''
-            cached_hash_dict = None
-            full_path = os.path.join(path, file_name)
-            md5 = hash_file(full_path)
-            cached_hash_dict = self.file_hash_cache[file_name]
-            try:
-                cached_hash_dict[md5]
-            except KeyError:
-                pass
-            return (None, md5)  # (f_uid, hash)
-        """
-
         recordable_events = ['IPS_CALL_BEGIN', 'IPS_STAGE_INPUTS', 'IPS_STAGE_OUTPUTS', 'IPS_CALL_END']
         recordable_mpo_activities = ['IPS_CALL_BEGIN']
         comment = event_data['comment']
@@ -381,23 +350,6 @@ class PortalBridge(Component):
                 (_, path, files) = o.groups()
                 glist = [glob.glob(os.path.join(path, f)) for f in files.split()]
                 for file_name in [os.path.basename(f) for f in itertools.chain(*glist)]:
-                    """
-                    (f_uid, f_hash) = get_file_uid(path, file_name)
-                    f_uid = False
-                    if f_uid:
-                        mpo_data_obj = self.mpo.add(workflow_ID=sim_data.mpo_wid,
-                                                    parentobj_ID=sim_data.mpo_wid['uid'],
-                                                    name=file_name,
-                                                    desc="An input file",
-                                                    uri='file:' + os.path.join(path, file_name),
-                                                    uid=f_uid)
-                    else:
-                        mpo_data_obj = self.mpo.add(workflow_ID=sim_data.mpo_wid,
-                                                    parentobj_ID=sim_data.mpo_wid['uid'],
-                                                    name=file_name,
-                                                    desc="An input file",
-                                                    uri='file:' + os.path.join(path, file_name))
-                    """
                     mpo_data_obj = mpo_add_file(sim_data.mpo_wid,
                                                 sim_data.mpo_wid['uid'],
                                                 os.path.join(path, file_name),
@@ -486,12 +438,11 @@ class PortalBridge(Component):
             sim_data.sim_root = os.environ['IPS_INITIAL_CWD']
         sim_log_dir = os.path.join(sim_data.sim_root, 'simulation_log')
         try:
-            os.makedirs(sim_log_dir)
+            os.makedirs(sim_log_dir, exist_ok=True)
         except OSError as oserr:
-            if oserr.errno != 17:
-                self.services.exception('Error creating Simulation Log directory %s : %d %s' %
-                                        (sim_log_dir, oserr.errno, oserr.strerror))
-                raise
+            self.services.exception('Error creating Simulation Log directory %s : %d %s' %
+                                    (sim_log_dir, oserr.errno, oserr.strerror))
+            raise
 
         sim_data.monitor_file_name = os.path.join(sim_log_dir,
                                                   sim_data.sim_name + '_' + sim_data.portal_runid + '.eventlog')
@@ -505,7 +456,7 @@ class PortalBridge(Component):
         json_fname = sim_data.monitor_file_name.replace('eventlog', 'json')
         sim_data.json_monitor_file = open(json_fname, 'w')
 
-        if self.mpo:
+        if self.mpo:  # pragma: no cover
             try:
                 sim_data.mpo_wid = self.mpo.init(name="SWIM Workflow " + os.environ["USER"],
                                                  desc=sim_data.sim_name,
@@ -519,4 +470,3 @@ class PortalBridge(Component):
                 sim_data.mpo_steps = [sim_data.mpo_wid['uid']]
 
         self.sim_map[sim_data.sim_name] = sim_data
-        return
