@@ -1,5 +1,5 @@
 # -------------------------------------------------------------------------------
-# Copyright 2006-2020 UT-Battelle, LLC. See LICENSE for more information.
+# Copyright 2006-2021 UT-Battelle, LLC. See LICENSE for more information.
 # -------------------------------------------------------------------------------
 # local version
 import os
@@ -84,11 +84,7 @@ class ResourceManager:
 
         rfile_name = os.path.join(self.CM.sim_map[self.CM.fwk_sim_name].sim_root, "resource_usage")
         # SIMYAN: try to safely make the directory...
-        try:
-            os.makedirs(self.CM.sim_map[self.CM.fwk_sim_name].sim_root)
-        except OSError as oserr:
-            if oserr.errno != 17:
-                raise
+        os.makedirs(self.CM.sim_map[self.CM.fwk_sim_name].sim_root, exist_ok=True)
 
         self.reporting_file = open(rfile_name, "w")
 
@@ -97,7 +93,7 @@ class ResourceManager:
         # -------------------------------
         if cmd_nodes != 0 and cmd_ppn != 0:
             # use cmd resource specification
-            self.host = "override_%s" % os.environ['HOST']
+            self.host = "override_%s" % self.CM.get_platform_parameter('HOST')
             self.max_ppn = cmd_ppn
             self.cores_per_node = int(cmd_ppn)
             self.ppn = int(cmd_ppn)
@@ -276,7 +272,7 @@ class ResourceManager:
           * *max_ppn*: processes that can be launched
           * *accurateNodes*: ``True`` if *nodes* uses the actual names of the nodes, ``False`` otherwise.
 
-        Aguments:
+        Arguments:
 
           * *nproc*: the number of requested processes (int)
           * *comp_id*: component identifier, must be unique with respect to the framework (string)
@@ -449,7 +445,7 @@ class ResourceManager:
         except Exception:
             self.fwk.exception("problem in RM.check_whole_node_cap")
             raise
-        # check to see if it is possible to satify the request
+        # check to see if it is possible to satisfy the request
         tot_cap = 0
         for n in list(self.nodes.values()):
             tot_cap += min([ppn, n.total_cores])
@@ -493,7 +489,7 @@ class ResourceManager:
         except Exception:
             self.fwk.exception("problem in RM.check_whole_sock_cap")
             raise
-        # check to see if it is possible to satify the request
+        # check to see if it is possible to satisfy the request
         tot_cap = 0
         for n in list(self.nodes.values()):
             tot_cap += min([ppn, n.total_cores])
@@ -540,7 +536,7 @@ class ResourceManager:
         except Exception:
             self.fwk.exception("problem in RM.check_core_cap")
             raise
-        # check to see if it is possible to satify the request
+        # check to see if it is possible to satisfy the request
         tot_cap = 0
         for n in list(self.nodes.values()):
             tot_cap += min([ppn, n.total_cores])
