@@ -1,11 +1,12 @@
 # -------------------------------------------------------------------------------
-# Copyright 2006-2020 UT-Battelle, LLC. See LICENSE for more information.
+# Copyright 2006-2021 UT-Battelle, LLC. See LICENSE for more information.
 # -------------------------------------------------------------------------------
 """IPS Services"""
 import sys
 import queue
 import os
 import subprocess
+import threading
 
 import time
 import shutil
@@ -13,8 +14,10 @@ import logging
 import logging.handlers
 import signal
 import glob
+import json
 import weakref
 import inspect
+from operator import itemgetter
 from . import messages, ipsutil, component
 from .configobj import ConfigObj
 from .cca_es_spec import initialize_event_service
@@ -27,67 +30,89 @@ def launch(binary, task_name, working_dir, *args, **keywords):
     input to :meth:`dask.distributed.Client.submit`.
 
     """
+    from dask.distributed import get_worker  # pylint: disable=import-outside-toplevel
+
+    worker = get_worker()
+    if not hasattr(worker, 'lock'):
+        worker.lock = threading.Lock()
+
+    worker_name = ''.join(c for c in get_worker().name if c.isalnum())
+
+    start_time = time.time()
     os.chdir(working_dir)
-    task_stdout = sys.stdout
+
+    worker_event_log = sys.stdout
     try:
-        log_filename = keywords["logfile"]
-    except KeyError:
+        event_logfile = keywords["worker_event_logfile"].format(worker_name)
+    except (KeyError, AttributeError):
         pass
     else:
-        task_stdout = open(log_filename, "w")
-
-    task_stderr = subprocess.STDOUT
-    try:
-        err_filename = keywords["errfile"]
-    except KeyError:
-        pass
-    else:
-        try:
-            task_stderr = open(err_filename, "w")
-        except Exception:
-            pass
-
-    task_env = {}
-    try:
-        task_env = keywords["task_env"]
-    except Exception:
-        pass
-    new_env = os.environ.copy()
-    new_env.update(task_env)
-
-    timeout = 1.e9
-    try:
-        timeout = float(keywords["timeout"])
-    except Exception:
-        pass
-
-    print(f"Task {task_name} timeout = {timeout}")
+        worker_event_log = open(event_logfile, 'a')
 
     ret_val = None
     if isinstance(binary, str):
+        task_stdout = sys.stdout
+        try:
+            log_filename = keywords["logfile"]
+        except KeyError:
+            pass
+        else:
+            task_stdout = open(log_filename, "w")
+
+        task_stderr = subprocess.STDOUT
+        try:
+            err_filename = keywords["errfile"]
+        except KeyError:
+            pass
+        else:
+            try:
+                task_stderr = open(err_filename, "w")
+            except OSError:
+                pass
+
+        task_env = keywords.get("task_env", {})
+        new_env = os.environ.copy()
+        new_env.update(task_env)
+
+        timeout = float(keywords.get("timeout", 1.e9))
+
         cmd = f"{binary} {' '.join(map(str, args))}"
+        with worker.lock:
+            print(json.dumps({"event_type": "IPS_LAUNCH_DASK_TASK", "event_time": time.time(),
+                              "event_comment": f"task_name = {task_name}, Target = {cmd}"}),
+                  file=worker_event_log)
+
         cmd_lst = cmd.split()
         process = subprocess.Popen(cmd_lst, stdout=task_stdout,
                                    stderr=task_stderr,
                                    cwd=working_dir,
                                    preexec_fn=os.setsid,
                                    env=new_env)
-        start = time.time()
-        while time.time() - start < timeout:
-            print(f"Task {task_name} going to sleep")
-            time.sleep(1.0)
-            print(f"Task {task_name} Woke up")
-            ret_val = process.poll()
-            if ret_val is None:
-                continue
-            else:
-                break
-        else:               # Time out for process execution
-            print(f"Task {task_name} timed out after {timeout} Seconds")
+        try:
+            ret_val = process.wait(timeout)
+            finish_time = time.time()
+            with worker.lock:
+                print(json.dumps({"event_type": "IPS_TASK_END", "event_time": finish_time,
+                                  "event_comment": f"task_name = {task_name}, elasped time = {finish_time - start_time:.2f}s"}),
+                      file=worker_event_log)
+        except subprocess.TimeoutExpired:
+            with worker.lock:
+                print(json.dumps({"event_type": "IPS_TASK_END", "event_time": time.time(),
+                                  "event_comment": f"task_name = {task_name}, timed-out after {timeout}s"}),
+                      file=worker_event_log)
             os.killpg(process.pid, signal.SIGKILL)
             ret_val = -1
     else:
+        with worker.lock:
+            print(json.dumps({"event_type": "IPS_LAUNCH_DASK_TASK", "event_time": time.time(),
+                              "event_comment": f"task_name = {task_name}, Target = {binary.__name__}({','.join(map(str, args))})"}),
+                  file=worker_event_log)
         ret_val = binary(*args)
+        finish_time = time.time()
+        with worker.lock:
+            print(json.dumps({"event_type": "IPS_TASK_END", "event_time": finish_time,
+                              "event_comment": f"task_name = {task_name}, elasped time = {finish_time - start_time:.2f}s"}),
+                  file=worker_event_log)
 
     return task_name, ret_val
 
@@ -207,10 +232,7 @@ class ServicesProxy:
                 self.fwk.exception("Bad 'NODE_ALLOCATION_MODE' value %s" % pn_compconf)
                 raise Exception("Bad 'NODE_ALLOCATION_MODE' value %s")
         except Exception:
-            if self.sim_conf['NODE_ALLOCATION_MODE'] == 'SHARED':
-                self.shared_nodes = True
-            else:
-                self.shared_nodes = False
+            self.shared_nodes = self.sim_conf['NODE_ALLOCATION_MODE'] == 'SHARED'
 
         # ------------------
         # set component ppn
@@ -306,15 +328,7 @@ class ServicesProxy:
             # get new messages, block until something interesting comes along
             responses = self._get_incoming_responses(block)
             for r in responses:
-                # time to die!
-                if r.__class__ == messages.ExitMessage:
-                    self.debug('%s Exiting', str(self.component_ref.component_id))
-                    if r.status == messages.Message.SUCCESS:
-                        sys.exit(0)
-                    else:
-                        sys.exit(1)
-                # response to my message
-                elif r.__class__ == messages.ServiceResponseMessage:
+                if r.__class__ == messages.ServiceResponseMessage:
                     if (r.request_msg_id not in
                             list(self.incomplete_calls.keys())):
                         self.error('Mismatched service response msg_id %s',
@@ -387,7 +401,8 @@ class ServicesProxy:
                             eventType='',
                             comment='',
                             ok='True',
-                            state='Running'):
+                            state='Running',
+                            event_time=None):
         """
         Construct and send an event populated with the component's
         information, *eventType*, *comment*, *ok*, *state*, and a wall time
@@ -399,7 +414,9 @@ class ServicesProxy:
                                         self.component_ref.NAME])
         portal_data['eventtype'] = eventType
         portal_data['ok'] = ok
-        portal_data['walltime'] = '%.2f' % (time.time() - self.component_ref.start_time)
+        if event_time is None:
+            event_time = time.time()
+        portal_data['walltime'] = '%.2f' % (event_time - self.component_ref.start_time)
         portal_data['state'] = state
         portal_data['comment'] = comment
         if self.monitor_url:
@@ -593,6 +610,10 @@ class ServicesProxy:
         :rtype: int
 
         """
+        if not isinstance(binary, str):
+            self.exception('Error in launch_task: task binary of wrong type, expected str but found %s', type(binary).__name__)
+            raise ValueError(f"task binary of wrong type, expected str but found {type(binary).__name__}")
+
         args = tuple(str(a) for a in args)
         tokens = binary.split()
         if len(tokens) > 1:
@@ -612,21 +633,8 @@ class ServicesProxy:
         block = keywords.get('block', True)
         tag = keywords.get('tag', 'None')
 
-        try:
-            whole_nodes = keywords['whole_nodes']
-        except Exception:
-            if self.shared_nodes:
-                whole_nodes = False
-            else:
-                whole_nodes = True
-
-        try:
-            whole_socks = keywords['whole_sockets']
-        except Exception:
-            if self.shared_nodes:
-                whole_socks = False
-            else:
-                whole_socks = True
+        whole_nodes = keywords.get('whole_nodes', not self.shared_nodes)
+        whole_socks = keywords.get('whole_sockets', not self.shared_nodes)
 
         try:
             # SIMYAN: added working_dir to component method invocation
@@ -707,25 +715,13 @@ class ServicesProxy:
         queued_tasks = task_pool.queued_tasks
         submit_dict = {}
         for (task_name, task) in queued_tasks.items():
-            task_ppn = self.ppn
-            try:
-                task_ppn = task.keywords['task_ppn']
-            except Exception:
-                pass
-            try:
-                wnodes = task.keywords['whole_nodes']
-            except Exception:
-                if self.shared_nodes:
-                    wnodes = False
-                else:
-                    wnodes = True
-            try:
-                wsocks = task.keywords['whole_sockets']
-            except Exception:
-                if self.shared_nodes:
-                    wsocks = False
-                else:
-                    wsocks = True
+            if not isinstance(task.binary, str):
+                self.exception('Error initiating task pool %s: task %s binary of wrong type, expected str but found %s',
+                               task_pool_name, task_name, type(task.binary).__name__)
+                raise ValueError(f"task {task_name} binary of wrong type, expected str but found {type(task.binary).__name__}")
+            task_ppn = task.keywords.get('task_ppn', self.ppn)
+            wnodes = task.keywords.get('whole_nodes', not self.shared_nodes)
+            wsocks = task.keywords.get('whole_sockets', not self.shared_nodes)
             submit_dict[task_name] = (task.nproc, task.working_dir,
                                       task.binary, task.args,
                                       task_ppn, wnodes, wsocks)
@@ -744,23 +740,12 @@ class ServicesProxy:
                 time.sleep(launch_interval)
             task = queued_tasks[task_name]
             (task_id, command, env_update) = allocated_tasks[task_name]
-            tag = 'None'
-            try:
-                tag = task.keywords['tag']
-            except KeyError:
-                pass
 
-            log_filename = None
-            try:
-                log_filename = task.keywords['logfile']
-            except KeyError:
-                pass
+            tag = task.keywords.get('tag', 'None')
 
-            timeout = 1.e9
-            try:
-                timeout = task.keywords["timeout"]
-            except KeyError:
-                pass
+            log_filename = task.keywords.get('logfile')
+
+            timeout = task.keywords.get("timeout", 1.e9)
 
             task_stdout = sys.stdout
             if log_filename:
@@ -844,9 +829,10 @@ class ServicesProxy:
         """
         Kill all tasks associated with this component.
         """
-        while len(self.task_map) > 0:
+        task_id_list = list(self.task_map)
+        for task_id in task_id_list:
             try:
-                self.kill_task(self.task_map[0])
+                self.kill_task(task_id)
             except Exception:
                 raise
 
@@ -890,7 +876,7 @@ class ServicesProxy:
         :param task_id: task ID (PID)
         :type task_id: int
 
-        :param timeout: maximum time to wait for task to finsih, default -1 (no timeout)
+        :param timeout: maximum time to wait for task to finish, default -1 (no timeout)
         :type timeout: float
 
         :param delay: time to wait before checking if task has timed-out
@@ -944,7 +930,7 @@ class ServicesProxy:
         :param task_id_list: list of task_id's (PID's) to wait until completed
         :type task_id_list: list of int
 
-        :param block: if to wait until all task finsh
+        :param block: if to wait until all task finish
         :type block: bool
 
         :return: dict of task_id and return value
@@ -978,10 +964,10 @@ class ServicesProxy:
         Return the value of the configuration parameter ``param``.  Raise
         exception if not found and silent is False.
 
-        :param param: The paramater requested from simulation config
+        :param param: The parameter requested from simulation config
         :type param: str
 
-        :param silent: If True and paramater isn't found then exception is not raised, default False
+        :param silent: If True and parameter isn't found then exception is not raised, default False
         :type silent: bool
 
         :return: dictionary of given parameter from configuration
@@ -1007,7 +993,7 @@ class ServicesProxy:
         :meth:`ipsframework.configurationManager.ConfigurationManager.set_config_parameter`
         to change the parameter.
 
-        :param param: The paramater requested from simulation config
+        :param param: The parameter requested from simulation config
         :type param: str
 
         :param value: The value to set the parameter
@@ -1077,7 +1063,7 @@ class ServicesProxy:
         Possible *MODE* options are:
 
         ALL:
-            Checkpint everytime the call is made (equivalent to always setting
+            Checkpint every time the call is made (equivalent to always setting
             Force =True)
         WALLTIME_REGULAR:
             checkpoints are saved upon invocation of the service call
@@ -1282,7 +1268,7 @@ class ServicesProxy:
 
         File are copied using :func:`ipsframework.ipsutil.copyFiles`.
 
-        :param input_file_list: input files can space seperated string or iterable of strings
+        :param input_file_list: input files can space separated string or iterable of strings
         :type input_file_list: str or Iterable of str
         """
         start_time = time.time()
@@ -1316,11 +1302,7 @@ class ServicesProxy:
                 input_dir = old_conf[c]['INPUT_DIR']
                 input_files = old_conf[c]['INPUT_FILES']
                 input_target_dir = os.path.join(os.getcwd(), c)
-                try:
-                    os.mkdir(input_target_dir)
-                except OSError as e:
-                    if e.errno != 17:
-                        raise
+                os.makedirs(input_target_dir, exist_ok=True)
                 try:
                     ipsutil.copyFiles(input_dir, input_files, input_target_dir)
                 except Exception as e:
@@ -1429,16 +1411,15 @@ class ServicesProxy:
                                   'simulation_results',
                                   'plasma_state')
         try:
-            os.makedirs(plasma_dir)
+            os.makedirs(plasma_dir, exist_ok=True)
         except OSError as e:
-            if e.errno != 17:
-                self._send_monitor_event('IPS_STAGE_OUTPUTS',
-                                         'Files = ' + str(file_list) +
-                                         ' Exception raised : ' + e.strerror,
-                                         ok='False')
-                self.exception('Error creating directory %s : %d-%s',
-                               plasma_dir, e.errno, e.strerror)
-                raise
+            self._send_monitor_event('IPS_STAGE_OUTPUTS',
+                                     'Files = ' + str(file_list) +
+                                     ' Exception raised : ' + e.strerror,
+                                     ok='False')
+            self.exception('Error creating directory %s : %d-%s',
+                           plasma_dir, e.errno, e.strerror)
+            raise
 
         all_plasma_files = []
         if save_plasma_state:
@@ -1485,12 +1466,11 @@ class ServicesProxy:
 
         symlink_dir = os.path.join(sim_root, out_root, self.full_comp_id)
         try:
-            os.makedirs(symlink_dir)
+            os.makedirs(symlink_dir, exist_ok=True)
         except OSError as e:
-            if e.errno != 17:
-                self.exception('Error creating directory %s : %s',
-                               symlink_dir, e.strerror)
-                raise
+            self.exception('Error creating directory %s : %s',
+                           symlink_dir, e.strerror)
+            raise
 
         all_files = sum([glob.glob(f) for f in file_list], [])
 
@@ -1766,12 +1746,14 @@ class ServicesProxy:
 
     def send_portal_event(self,
                           event_type="COMPONENT_EVENT",
-                          event_comment=""):
+                          event_comment="",
+                          event_time=None):
         """
         Send event to web portal.
         """
         return self._send_monitor_event(eventType=event_type,
-                                        comment=event_comment)
+                                        comment=event_comment,
+                                        event_time=event_time)
 
     def log(self, msg, *args):
         """
@@ -1807,7 +1789,7 @@ class ServicesProxy:
         """
         Produce **exception** message in simulation log file. See :func:`logging.exception` for usage.
         """
-        self.logger.exception(msg, *args, exc_info=False)
+        self.logger.exception(msg, *args)
 
     def critical(self, msg, *args):
         """
@@ -1834,17 +1816,19 @@ class ServicesProxy:
                                   *args, keywords=keywords)
 
     def submit_tasks(self, task_pool_name, block=True, use_dask=False, dask_nodes=1,
-                     dask_ppn=None, launch_interval=0.0):
+                     dask_ppn=None, launch_interval=0.0, use_shifter=False):
         """
         Launch all unfinished tasks in task pool *task_pool_name*.  If *block* is ``True``,
         return when all tasks have been launched.  If *block* is ``False``, return when all
         tasks that can be launched immediately have been launched.  Return number of tasks
         submitted.
+
+        Optionally, dask can be used to schedule and run the task pool.
         """
         start_time = time.time()
         self._send_monitor_event('IPS_TASK_POOL_BEGIN', 'task_pool = %s ' % task_pool_name)
         task_pool: TaskPool = self.task_pools[task_pool_name]
-        retval = task_pool.submit_tasks(block, use_dask, dask_nodes, dask_ppn, launch_interval)
+        retval = task_pool.submit_tasks(block, use_dask, dask_nodes, dask_ppn, launch_interval, use_shifter)
         self._send_monitor_event('IPS_TASK_POOL_END', 'task_pool = %s  elapsed time = %.2f S' %
                                  (task_pool_name, time.time() - start_time))
         return retval
@@ -1864,13 +1848,11 @@ class ServicesProxy:
         task_pool.terminate_tasks()
         del self.task_pools[task_pool_name]
 
-    def create_sub_workflow(self, sub_name, config_file, override=None, input_dir=None):
+    def create_sub_workflow(self, sub_name, config_file, override={}, input_dir=None):
         """Create sub-workflow
 
         """
 
-        if not override:
-            override = {}
         if sub_name in list(self.sub_flows.keys()):
             self.exception("Duplicate sub flow name")
             raise Exception("Duplicate sub flow name")
@@ -1948,15 +1930,14 @@ class TaskPool:
     distributed = None
     try:
         dask: dask = __import__("dask")
-        distributed: dask.distributed = __import__("dask.distributed",
-                                                   fromlist=[None])
+        distributed: dask.distributed = __import__("dask.distributed")
     except ImportError:
-        pass
+        dask = None
+        distributed = None
     else:
-        dask_scheduler = None
-        dask_worker = None
         dask_scheduler = shutil.which("dask-scheduler")
         dask_worker = shutil.which("dask-worker")
+        shifter = shutil.which("shifter")
         if not dask_scheduler or not dask_worker:
             dask = None
             distributed = None
@@ -2049,68 +2030,81 @@ class TaskPool:
         self.queued_tasks[task_name] = Task(task_name, nproc, working_dir, binary_fullpath, *args,
                                             **keywords["keywords"])
 
-    def submit_dask_tasks(self, block=True, dask_nodes=1, dask_ppn=None):
-        """Launch tasks in *queued_tasks* using dask.  Finished tasks are
-        handled before launching new ones.  If *block* is ``True``,
-        the number of tasks submited is returned after all tasks have
-        been launched and completed.  If *block* is ``False`` the
-        number of tasks that can immediately be launched is returned.
+    def submit_dask_tasks(self, block=True, dask_nodes=1, dask_ppn=None, use_shifter=False):
+        """Launch tasks in *queued_tasks* using dask.
 
-        :param block: If True then wait for task to complete, default True
+        :param block: Unused, this will always return after tasks are submitted
         :type block: bool
         :param dask_nodes: Number of task nodes, default 1
         :type dask_nodes: int
         :param dask_ppn:  Number of processes per node, default None
         :type dask_ppn: int
+        :param use_shifter:  Option to launch dask scheduler and workers in shifter container
+        :type use_shifter: bool
         """
         services: ServicesProxy = self.services
         self.dask_file_name = os.path.join(os.getcwd(),
                                            f".{self.name}_dask_shed_{time.time()}.json")
-        self.dask_sched_pid = subprocess.Popen([self.dask_scheduler, "--no-dashboard",
-                                                "--scheduler-file", self.dask_file_name, "--port", "0"]).pid
+        if use_shifter:
+            self.dask_sched_pid = subprocess.Popen([self.shifter, "dask-scheduler", "--no-dashboard",
+                                                    "--scheduler-file", self.dask_file_name, "--port", "0"]).pid
+        else:
+            self.dask_sched_pid = subprocess.Popen([self.dask_scheduler, "--no-dashboard",
+                                                    "--scheduler-file", self.dask_file_name, "--port", "0"]).pid
 
         dask_nodes = 1 if dask_nodes is None else dask_nodes
         if services.get_config_param("MPIRUN") == "eval":
             dask_nodes = 1
 
         nthreads = dask_ppn if dask_ppn else services.get_config_param("PROCS_PER_NODE")
-        self.dask_workers_tid = services.launch_task(dask_nodes, os.getcwd(),
-                                                     self.dask_worker,
-                                                     "--scheduler-file",
-                                                     self.dask_file_name,
-                                                     "--nprocs", 1,
-                                                     "--nthreads", nthreads,
-                                                     "--no-dashboard",
-                                                     "--preload", self.services.dask_preload,
-                                                     task_ppn=1)
+        if use_shifter:
+            self.dask_workers_tid = services.launch_task(dask_nodes, os.getcwd(),
+                                                         self.shifter,
+                                                         "dask-worker",
+                                                         "--scheduler-file",
+                                                         self.dask_file_name,
+                                                         "--nprocs", 1,
+                                                         "--nthreads", nthreads,
+                                                         "--no-dashboard",
+                                                         "--preload", self.services.dask_preload,
+                                                         task_ppn=1)
+        else:
+            self.dask_workers_tid = services.launch_task(dask_nodes, os.getcwd(),
+                                                         self.dask_worker,
+                                                         "--scheduler-file",
+                                                         self.dask_file_name,
+                                                         "--nprocs", 1,
+                                                         "--nthreads", nthreads,
+                                                         "--no-dashboard",
+                                                         "--preload", self.services.dask_preload,
+                                                         task_ppn=1)
 
         self.dask_client = self.dask.distributed.Client(scheduler_file=self.dask_file_name)
+
+        try:
+            self.worker_event_logfile = services.sim_name + '_' + services.get_config_param("PORTAL_RUNID") + '_' + self.name + '_{}.json'
+        except KeyError:
+            # USE_PORTAL == False
+            self.worker_event_logfile = None
+
         launch.__module__ = "__main__"
         self.futures = []
-        for k, v in self.queued_tasks.items():
-            try:
-                log_filename = v.keywords["logfile"]
-            except KeyError:
-                pass
-            else:
-                if not os.path.isabs(log_filename):
-                    full_path = os.path.join(os.getcwd(), log_filename)
-                    v.keywords["logfile"] = full_path
-
+        for task_name, task in self.queued_tasks.items():
             self.futures.append(self.dask_client.submit(launch,
-                                                        v.binary,
-                                                        k,
-                                                        v.working_dir,
-                                                        *v.args,
-                                                        **v.keywords))
+                                                        task.binary,
+                                                        task_name,
+                                                        task.working_dir,
+                                                        *task.args,
+                                                        **task.keywords,
+                                                        worker_event_logfile=self.worker_event_logfile))
         self.active_tasks = self.queued_tasks
         self.queued_tasks = {}
         return len(self.futures)
 
-    def submit_tasks(self, block=True, use_dask=False, dask_nodes=1, dask_ppn=None, launch_interval=0.0):
+    def submit_tasks(self, block=True, use_dask=False, dask_nodes=1, dask_ppn=None, launch_interval=0.0, use_shifter=False):
         """Launch tasks in *queued_tasks*.  Finished tasks are handled before
         launching new ones.  If *block* is ``True``, the number of
-        tasks submited is returned after all tasks have been launched
+        tasks submitted is returned after all tasks have been launched
         and completed.  If *block* is ``False`` the number of tasks
         that can immediately be launched is returned.
 
@@ -2127,12 +2121,23 @@ class TaskPool:
         :type dask_ppn: int
         :param launch_internal: time to wait between launching tasks, default 0.0
         :type launch_internal: float
+        :param use_shifter:  Option to launch dask scheduler and workers in shifter container
+        :type use_shifter: bool
 
         """
 
-        if TaskPool.dask and self.serial_pool and use_dask:
-            self.dask_pool = True
-            return self.submit_dask_tasks(block, dask_nodes, dask_ppn)
+        if use_dask:
+            if TaskPool.dask and self.serial_pool:
+                self.dask_pool = True
+                if use_shifter and not self.shifter:
+                    self.services.exception("Requested to run dask within shifter but shifter not available")
+                    raise Exception("shifter not found")
+                else:
+                    return self.submit_dask_tasks(block, dask_nodes, dask_ppn, use_shifter)
+            elif not TaskPool.dask:
+                self.services.warning("Requested use_dask but cannot because import dask failed")
+            elif not self.serial_pool:
+                self.services.warning("Requested use_dask but cannot because multiple processors requested")
 
         submit_count = 0
         # Make sure any finished tasks are handled before attempting to submit
@@ -2162,9 +2167,37 @@ class TaskPool:
         :rtype: dict
         """
         result = self.dask_client.gather(self.futures)
+        worker_names = [''.join(c for c in worker['name'] if c.isalnum()) for worker in self.dask_client.scheduler_info()['workers'].values()]
         self.dask_client.shutdown()
         self.dask_client.close()
         time.sleep(1)
+        if self.worker_event_logfile is not None:
+            try:
+                events = []
+                for worker in worker_names:
+                    filename = self.worker_event_logfile.format(worker)
+                    try:
+                        lines = open(filename).readlines()
+                    except IOError:
+                        self.services.exception('Error opening dask worker log file: %s', filename)
+                    else:
+                        # convert to dict and sort by event_time
+                        for line in lines:
+                            try:
+                                events.append(json.loads(line.strip()))
+                            except json.decoder.JSONDecodeError:
+                                self.services.exception('Error reading line %s from dask worker log file: %s', line.strip(), filename)
+
+                events.sort(key=itemgetter('event_time'))
+                for event in events:
+                    self.services.send_portal_event(**event)
+            except Exception as e:
+                # If it fails for any other reason, make sure we can continue
+                self.services.exception('Error while reading dask worker log files: %s', str(e))
+            else:
+                for worker in worker_names:
+                    os.remove(self.worker_event_logfile.format(worker))
+
         self.finished_tasks = {}
         self.active_tasks = {}
         self.services.wait_task(self.dask_workers_tid)
